==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/kinetics/saw/SawBlock.java <==
package com.zurrtum.create.content.kinetics.saw;

import com.zurrtum.create.*;
import com.zurrtum.create.catnip.placement.IPlacementHelper;
import com.zurrtum.create.catnip.placement.PlacementHelpers;
import com.zurrtum.create.catnip.placement.PlacementOffset;
import com.zurrtum.create.content.kinetics.base.DirectionalAxisKineticBlock;
import com.zurrtum.create.content.kinetics.drill.DrillBlock;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.infrastructure.items.ItemInventoryProvider;
import java.util.List;
import java.util.function.Predicate;
import net.minecraft.class_10;
import net.minecraft.class_10774;
import net.minecraft.class_1263;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1297;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_2350.class_2352;
import net.minecraft.class_238;
import net.minecraft.class_2415;
import net.minecraft.class_2470;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3726;
import net.minecraft.class_3965;
import net.minecraft.class_4538;

public class SawBlock extends DirectionalAxisKineticBlock implements IBE<SawBlockEntity>, ItemInventoryProvider<SawBlockEntity> {
    public static final class_2746 FLIPPED = class_2746.method_11825("flipped");

    private static final int placementHelperId = PlacementHelpers.register(new PlacementHelper());

    public SawBlock(class_2251 properties) {
        super(properties);
    }

    @Override
    public class_1263 getInventory(class_1936 world, class_2338 pos, class_2680 state, SawBlockEntity blockEntity, class_2350 context) {
        return blockEntity.inventory;
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        super.method_9515(builder.method_11667(FLIPPED));
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 stateForPlacement = super.method_9605(context);
        class_2350 facing = stateForPlacement.method_11654(FACING);
        return stateForPlacement.method_11657(
            FLIPPED,
            facing.method_10166() == class_2351.field_11052 && context.method_8042().method_10171() == class_2352.field_11056
        );
    }

    @Override
    public class_2680 getRotatedBlockState(class_2680 originalState, class_2350 targetedFace) {
        class_2680 newState = super.getRotatedBlockState(originalState, targetedFace);
        if (newState.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;
        if (targetedFace.method_10166() != class_2351.field_11052)
            return newState;
        if (!originalState.method_11654(AXIS_ALONG_FIRST_COORDINATE))
            newState = newState.method_28493(FLIPPED);
        return newState;
    }

    @Override
    public class_2680 method_9598(class_2680 state, class_2470 rot) {
        class_2680 newState = super.method_9598(state, rot);
        if (state.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;

        if (rot.ordinal() % 2 == 1 && (rot == class_2470.field_11463) != state.method_11654(AXIS_ALONG_FIRST_COORDINATE))
            newState = newState.method_28493(FLIPPED);
        if (rot == class_2470.field_11464)
            newState = newState.method_28493(FLIPPED);

        return newState;
    }

    @Override
    public class_2680 method_9569(class_2680 state, class_2415 mirrorIn) {
        class_2680 newState = super.method_9569(state, mirrorIn);
        if (state.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;

        boolean alongX = state.method_11654(AXIS_ALONG_FIRST_COORDINATE);
        if (alongX && mirrorIn == class_2415.field_11301)
            newState = newState.method_28493(FLIPPED);
        if (!alongX && mirrorIn == class_2415.field_11300)
            newState = newState.method_28493(FLIPPED);

        return newState;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 worldIn, class_2338 pos, class_3726 context) {
        return AllShapes.CASING_12PX.get(state.method_11654(FACING));
    }

    @Override
    protected class_1269 method_55765(
        class_1799 stack,
        class_2680 state,
        class_1937 level,
        class_2338 pos,
        class_1657 player,
        class_1268 hand,
        class_3965 hitResult
    ) {
        IPlacementHelper placementHelper = PlacementHelpers.get(placementHelperId);
        if (!player.method_5715() && player.method_7294()) {
            if (placementHelper.matchesItem(stack) && placementHelper.getOffset(player, level, state, pos, hitResult)
                .placeInWorld(level, (class_1747) stack.method_7909(), player, hand).method_23665())
                return class_1269.field_5812;
        }

        if (player.method_7325() || !stack.method_7960())
            return class_1269.field_52423;
        if (state.method_61767(FACING, class_2350.field_11039) != class_2350.field_11036)
            return class_1269.field_52423;

        return onBlockEntityUseItemOn(
            level, pos, be -> {
                if (!level.field_9236) {
                    for (int i = 0, size = be.inventory.method_5439(); i < size; i++) {
                        class_1799 heldItemStack = be.inventory.method_5438(i);
                        if (heldItemStack.method_7960()) {
                            continue;
                        }
                        player.method_31548().method_7398(heldItemStack);
                    }
                }
                be.inventory.method_5448();
                be.notifyUpdate();
                return class_1269.field_5812;
            }
        );
    }

    @Override
    public void method_9548(class_2680 state, class_1937 worldIn, class_2338 pos, class_1297 entityIn, class_10774 handler) {
        if (worldIn.field_9236 || entityIn instanceof class_1542)
            return;
        if (!new class_238(pos).method_1011(.1f).method_994(entityIn.method_5829()))
            return;
        withBlockEntityDo(
            worldIn, pos, be -> {
                if (be.getSpeed() == 0)
                    return;
                entityIn.method_64397((class_3218) worldIn, AllDamageSources.get(worldIn).saw, (float) DrillBlock.getDamage(be.getSpeed()));
            }
        );
    }

    @Override
    public void method_9502(class_1922 worldIn, class_1297 entityIn) {
        super.method_9502(worldIn, entityIn);
        if (!(entityIn instanceof class_1542))
            return;
        if (entityIn.method_37908().field_9236)
            return;

        class_2338 pos = entityIn.method_24515();
        withBlockEntityDo(
            entityIn.method_37908(), pos, be -> {
                if (be.getSpeed() == 0)
                    return;
                be.insertItem((class_1542) entityIn);
            }
        );
    }

    public static boolean isHorizontal(class_2680 state) {
        return state.method_11654(FACING).method_10166().method_10179();
    }

    @Override
    public class_2351 getRotationAxis(class_2680 state) {
        return isHorizontal(state) ? state.method_11654(FACING).method_10166() : super.getRotationAxis(state);
    }

    @Override
    public boolean hasShaftTowards(class_4538 world, class_2338 pos, class_2680 state, class_2350 face) {
        return isHorizontal(state) ? face == state.method_11654(FACING).method_10153() : super.hasShaftTowards(world, pos, state, face);
    }

    @Override
    public Class<SawBlockEntity> getBlockEntityClass() {
        return SawBlockEntity.class;
    }

    @Override
    public class_2591<? extends SawBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.SAW;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    private static class PlacementHelper implements IPlacementHelper {

        @Override
        public Predicate<class_1799> getItemPredicate() {
            return stack -> stack.method_31574(AllItems.MECHANICAL_SAW);
        }

        @Override
        public Predicate<class_2680> getStatePredicate() {
            return state -> state.method_27852(AllBlocks.MECHANICAL_SAW);
        }

        @Override
        public PlacementOffset getOffset(class_1657 player, class_1937 world, class_2680 state, class_2338 pos, class_3965 ray) {
            List<class_2350> directions = IPlacementHelper.orderedByDistanceExceptAxis(
                pos,
                ray.method_17784(),
                state.method_11654(FACING).method_10166(),
                dir -> world.method_8320(pos.method_10093(dir)).method_45474()
            );

            if (directions.isEmpty())
                return PlacementOffset.fail();
            else {
                return PlacementOffset.success(
                    pos.method_10093(directions.getFirst()),
                    s -> s.method_11657(FACING, state.method_11654(FACING)).method_11657(AXIS_ALONG_FIRST_COORDINATE, state.method_11654(AXIS_ALONG_FIRST_COORDINATE))
                        .method_11657(FLIPPED, state.method_11654(FLIPPED))
                );
            }
        }

    }

}

==> create-fly-1.21.10-6.0.8-3-sources.jar: com/zurrtum/create/content/kinetics/saw/SawBlock.java <==
package com.zurrtum.create.content.kinetics.saw;

import com.zurrtum.create.*;
import com.zurrtum.create.catnip.placement.IPlacementHelper;
import com.zurrtum.create.catnip.placement.PlacementHelpers;
import com.zurrtum.create.catnip.placement.PlacementOffset;
import com.zurrtum.create.content.kinetics.base.DirectionalAxisKineticBlock;
import com.zurrtum.create.content.kinetics.drill.DrillBlock;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.infrastructure.items.ItemInventoryProvider;
import java.util.List;
import java.util.function.Predicate;
import net.minecraft.class_10;
import net.minecraft.class_10774;
import net.minecraft.class_1263;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1297;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_2350.class_2352;
import net.minecraft.class_238;
import net.minecraft.class_2415;
import net.minecraft.class_2470;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3726;
import net.minecraft.class_3965;
import net.minecraft.class_4538;

public class SawBlock extends DirectionalAxisKineticBlock implements IBE<SawBlockEntity>, ItemInventoryProvider<SawBlockEntity> {
    public static final class_2746 FLIPPED = class_2746.method_11825("flipped");

    private static final int placementHelperId = PlacementHelpers.register(new PlacementHelper());

    public SawBlock(class_2251 properties) {
        super(properties);
    }

    @Override
    public class_1263 getInventory(class_1936 world, class_2338 pos, class_2680 state, SawBlockEntity blockEntity, class_2350 context) {
        return blockEntity.inventory;
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        super.method_9515(builder.method_11667(FLIPPED));
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        class_2680 stateForPlacement = super.method_9605(context);
        class_2350 facing = stateForPlacement.method_11654(FACING);
        return stateForPlacement.method_11657(
            FLIPPED,
            facing.method_10166() == class_2351.field_11052 && context.method_8042().method_10171() == class_2352.field_11056
        );
    }

    @Override
    public class_2680 getRotatedBlockState(class_2680 originalState, class_2350 targetedFace) {
        class_2680 newState = super.getRotatedBlockState(originalState, targetedFace);
        if (newState.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;
        if (targetedFace.method_10166() != class_2351.field_11052)
            return newState;
        if (!originalState.method_11654(AXIS_ALONG_FIRST_COORDINATE))
            newState = newState.method_28493(FLIPPED);
        return newState;
    }

    @Override
    public class_2680 method_9598(class_2680 state, class_2470 rot) {
        class_2680 newState = super.method_9598(state, rot);
        if (state.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;

        if (rot.ordinal() % 2 == 1 && (rot == class_2470.field_11463) != state.method_11654(AXIS_ALONG_FIRST_COORDINATE))
            newState = newState.method_28493(FLIPPED);
        if (rot == class_2470.field_11464)
            newState = newState.method_28493(FLIPPED);

        return newState;
    }

    @Override
    public class_2680 method_9569(class_2680 state, class_2415 mirrorIn) {
        class_2680 newState = super.method_9569(state, mirrorIn);
        if (state.method_11654(FACING).method_10166() != class_2351.field_11052)
            return newState;

        boolean alongX = state.method_11654(AXIS_ALONG_FIRST_COORDINATE);
        if (alongX && mirrorIn == class_2415.field_11301)
            newState = newState.method_28493(FLIPPED);
        if (!alongX && mirrorIn == class_2415.field_11300)
            newState = newState.method_28493(FLIPPED);

        return newState;
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 worldIn, class_2338 pos, class_3726 context) {
        return AllShapes.CASING_12PX.get(state.method_11654(FACING));
    }

    @Override
    protected class_1269 method_55765(
        class_1799 stack,
        class_2680 state,
        class_1937 level,
        class_2338 pos,
        class_1657 player,
        class_1268 hand,
        class_3965 hitResult
    ) {
        IPlacementHelper placementHelper = PlacementHelpers.get(placementHelperId);
        if (!player.method_5715() && player.method_7294()) {
            if (placementHelper.matchesItem(stack) && placementHelper.getOffset(player, level, state, pos, hitResult)
                .placeInWorld(level, (class_1747) stack.method_7909(), player, hand).method_23665())
                return class_1269.field_5812;
        }

        if (player.method_7325() || !stack.method_7960())
            return class_1269.field_52423;
        if (state.method_61767(FACING, class_2350.field_11039) != class_2350.field_11036)
            return class_1269.field_52423;

        return onBlockEntityUseItemOn(
            level, pos, be -> {
                if (!level.method_8608()) {
                    for (int i = 0, size = be.inventory.method_5439(); i < size; i++) {
                        class_1799 heldItemStack = be.inventory.method_5438(i);
                        if (heldItemStack.method_7960()) {
                            continue;
                        }
                        player.method_31548().method_7398(heldItemStack);
                    }
                }
                be.inventory.method_5448();
                be.notifyUpdate();
                return class_1269.field_5812;
            }
        );
    }

    @Override
    public void method_9548(class_2680 state, class_1937 worldIn, class_2338 pos, class_1297 entityIn, class_10774 handler, boolean bl) {
        if (worldIn.method_8608() || entityIn instanceof class_1542)
            return;
        if (!new class_238(pos).method_1011(.1f).method_994(entityIn.method_5829()))
            return;
        withBlockEntityDo(
            worldIn, pos, be -> {
                if (be.getSpeed() == 0)
                    return;
                entityIn.method_64397((class_3218) worldIn, AllDamageSources.get(worldIn).saw, (float) DrillBlock.getDamage(be.getSpeed()));
            }
        );
    }

    @Override
    public void method_9502(class_1922 worldIn, class_1297 entityIn) {
        super.method_9502(worldIn, entityIn);
        if (!(entityIn instanceof class_1542))
            return;
        if (entityIn.method_73183().method_8608())
            return;

        class_2338 pos = entityIn.method_24515();
        withBlockEntityDo(
            entityIn.method_73183(), pos, be -> {
                if (be.getSpeed() == 0)
                    return;
                be.insertItem((class_1542) entityIn);
            }
        );
    }

    public static boolean isHorizontal(class_2680 state) {
        return state.method_11654(FACING).method_10166().method_10179();
    }

    @Override
    public class_2351 getRotationAxis(class_2680 state) {
        return isHorizontal(state) ? state.method_11654(FACING).method_10166() : super.getRotationAxis(state);
    }

    @Override
    public boolean hasShaftTowards(class_4538 world, class_2338 pos, class_2680 state, class_2350 face) {
        return isHorizontal(state) ? face == state.method_11654(FACING).method_10153() : super.hasShaftTowards(world, pos, state, face);
    }

    @Override
    public Class<SawBlockEntity> getBlockEntityClass() {
        return SawBlockEntity.class;
    }

    @Override
    public class_2591<? extends SawBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.SAW;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    private static class PlacementHelper implements IPlacementHelper {

        @Override
        public Predicate<class_1799> getItemPredicate() {
            return stack -> stack.method_31574(AllItems.MECHANICAL_SAW);
        }

        @Override
        public Predicate<class_2680> getStatePredicate() {
            return state -> state.method_27852(AllBlocks.MECHANICAL_SAW);
        }

        @Override
        public PlacementOffset getOffset(class_1657 player, class_1937 world, class_2680 state, class_2338 pos, class_3965 ray) {
            List<class_2350> directions = IPlacementHelper.orderedByDistanceExceptAxis(
                pos,
                ray.method_17784(),
                state.method_11654(FACING).method_10166(),
                dir -> world.method_8320(pos.method_10093(dir)).method_45474()
            );

            if (directions.isEmpty())
                return PlacementOffset.fail();
            else {
                return PlacementOffset.success(
                    pos.method_10093(directions.getFirst()),
                    s -> s.method_11657(FACING, state.method_11654(FACING)).method_11657(AXIS_ALONG_FIRST_COORDINATE, state.method_11654(AXIS_ALONG_FIRST_COORDINATE))
                        .method_11657(FLIPPED, state.method_11654(FLIPPED))
                );
            }
        }

    }

}
